unit bcParseDemo;

{
  TbcParser, mathematical expression parser component demo,
  shows many features of the component.
  Please read the bcParser.hlp for complete information.
  This demo does not show every way to use the component.

  by Suavi Ali Demir, 2000

  www.bestcode.com
  support@bestcode.com
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  bcParser, StdCtrls, Buttons;

type
  TForm1 = class(TForm)
    bbCalc: TBitBtn;
    lbFormula: TListBox;
    eFormula: TEdit;
    Label1: TLabel;
    labResult: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    eXVal: TEdit;
    eYVal: TEdit;
    bcParser: TbcParser;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure lbFormulaDblClick(Sender: TObject);
    procedure bbCalcClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
     lbFormula.Items.Add('SIN(3.14)+5^2+POW(2,7)-MAX(10,20)');
     lbFormula.Items.Add('{ [ SIN(X)^2 ] + COS(1) } * PI/2');
     lbFormula.Items.Add('( COSH(2.71)+ SINH( COS(0.5) * POW(0.2345, 0.67) )) - LOG(1-X^2+X^5)');
     lbFormula.Items.Add('(90+292*POW(X,0.31))*(1+0.025*LOG(Y))*(1-1*POW(Y+X,1.09))');
     lbFormula.Items.Add('25993.894*EXP(-1.53389*(X-32))*POW(X, 0.13547)*POW(Y, 0.38603)*EXP(-0.36222*Y)');
     eFormula.Text := lbFormula.Items.Strings[0];
     lbFormula.ItemIndex := 0;
end;

procedure TForm1.lbFormulaDblClick(Sender: TObject);
begin
     if not (lbFormula.ItemIndex = -1) then
          eFormula.Text := lbFormula.Items[lbFormula.ItemIndex];
end;

procedure TForm1.bbCalcClick(Sender: TObject);
var i: integer;
    count: integer;
begin
  bcParser.Expression := eFormula.Text; //assign the expression
  //nothing will happen now. Expression is not parsed yet.
  //This is just string assignment so far.
  try
       bcParser.X := StrToFloat(eXVal.Text); //variables are set.
       bcParser.Y := StrToFloat(eYVal.Text);
       labResult.Caption := FloatToStr(bcParser.Value); //might throw exception
       //Value property executes a function that
       //checks the dirty flag, and calls the Parse method if needed.
       ShowMessage(labResult.Caption); //display the result! :)
  except
     on e: Exception do
     ShowMessage(e.Message); //might show "div by zero" errors or "cannot parse" errors...
  end;

  //lets try another expression:
  bcParser.Expression := lbFormula.Items.Strings[4]; //try the last one in the listbox.
  //when the expression is assigned, Dirty flag is set to true.
  //Expression will be parsed once again when we get the Value property.

  //But if we just want to check for errors we can call the Parse method, instead
  //of Value method, so that we won't evaluate it at this moment.

  //This maybe necessary to check the validity of a user input formula.
  //We may have variable values that may cause div/0 that could interfere with
  //parser error detection.

  bcParser.X := 0; //parse will still parse even if evaluation
                   //could result in div/0
  bcParser.Y := 0;
  try
        bcParser.Parse; //create parse tree.
  except
     on e: Exception do
     ShowMessage(e.Message); //might show "cannot parse" errors...
  end;


  count:= GetTickCount;
  try
    for i:= 1 to 1000 do
    begin
       bcParser.X := i*2;
       bcParser.Y := i/2;
       //calling Value property repeatedly in the loop does not cause
       //repeated parsing, expression is already parsed and now it is evaluated quickly.
       labResult.Caption := FloatToStr(bcParser.Value);
     end;
  except
     on e: Exception do
     ShowMessage(e.Message);
  end;
  ShowMessage(FloatToStr(GetTickCount - count));

  //just for the sake of example, lets create and remove a variable:
  bcParser.CreateVar('T', 0.0); //create a variable named "t" with the initial value 0.0
  bcParser.CreateVar('U', 5);
  bcParser.DeleteVar('T'); //you can delete variables this way
  bcParser.DeleteVar('U');

  bcParser.CreateVar('Z', 10);
  //lets change the expression so that it contains our new variable Z:
  bcParser.Expression := lbFormula.Items.Strings[4] + ' / z';
  //now expression will be reparsed when we get the Value property:
  labResult.Caption := FloatToStr(bcParser.Value);

  //note that we need to delete the new variable Z here or when we
  //click the Calculate button, Parser will throw an exception saying that
  //variable Z already exists.
  bcParser.DeleteVar('Z'); //delete it now so that we can create it again later.
end;

end.
